/*
 * Copyright (c) 2001 David I. Bell
 * Permission is granted to use, distribute, or modify this source,
 * provided that this copyright notice remains intact.
 */

import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.JTable;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.SwingUtilities;

import java.awt.Dimension;
import java.awt.Color;


/**
 * An class which manages multiple connection listeners using Swing.
 */
public class SwingManager extends ConnectionManager
{
	/**
	 * The title for the window.
	 */
	public final static String TITLE = "Browser Watcher";


	//
	// The swing widgets which we use.
	//
	protected JFrame		mFrame;
	protected JPanel		mPanel;
	protected JScrollPane		mScrollPane;
	protected JTable		mTable;
	protected SwingConnectionModel	mModel;


	/**
	 * Initialize the connection manager in preparation for
	 * connection listeners.  We build a top level JFrame which
	 * displays a table of the connections.
	 */
	public void initialize()
	{
		//
		// Get some settings that we use.
		//
		String columnNames = Settings.getColumnNames();
		String columnWidths = Settings.getColumnWidths();
		int width = Settings.getWidth();
		int height = Settings.getHeight();

		if (width <= 0)
			width = 400;

		if (height <= 0)
			height = 100;

		//
		// Set the column names if it was specified.
		//
		if (columnNames != null)
			SwingListener.setColumnList(columnNames);

		//
		// Get our own model for a table.
		//
		mModel = new SwingConnectionModel(this);

		mTable = new JTable(mModel);

		mTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);

		if (columnWidths != null)
			setColumnWidths(columnWidths);

		JScrollPane mScrollPane = new JScrollPane(mTable);

		//
		// Allocate the top level frame and set its size.
		//
		mFrame = new JFrame(TITLE);

		mFrame.setSize(new Dimension(width, height));

		mFrame.getContentPane().add(mScrollPane);

		//
		// Show the window.
		//
		mFrame.show();
	}


	/**
	 * Set the column widths for the table.
	 * The widths are a colon-separated list of numbers.
	 */
	public void setColumnWidths(String columnWidths)
	{
		StringTokenizer token = new StringTokenizer(columnWidths, ":");

		//
		// Get the column model for the table and the column count.
		//
		TableColumnModel columnModel = mTable.getColumnModel();
		int columnCount = mTable.getColumnCount();

		//
		// Loop over all of the columns and integer values.
		// Stop when either one runs out.
		//
		int index = 0;

		while ((index < columnCount) && (token.hasMoreTokens()))
		{
			String widthString = token.nextToken();

			//
			// Convert the width into a number.
			//
			int width = 0;

			try
			{
				width = Integer.parseInt(widthString);
			}
			catch (Exception e)
			{
				throw new IllegalArgumentException(
					"Illegal width " + widthString);
			}

			//
			// Get the column for this index.
			//
			TableColumn column = columnModel.getColumn(index);

			//
			// If the width is positive then set it.
			//
			if (width > 0)
				column.setPreferredWidth(width);

			index++;
		}
	}


	/**
	 * Method called when the values of a row have changed.
	 * The supplied listener indicates the row that was changed,
	 * or null if the row is not known.
	 */
	public void changed(ConnectionListener listener)
	{
		//
		// Update the GUI in the swing thread.
		// TODO: Make more efficient.
		//
		SwingUtilities.invokeLater(new Runnable() {
			public void run()
			{
				mModel.fireTableDataChanged();
			}
		});
	}


	/**
	 * Free the connection listener for a connection.
	 * We implement a timeout for this activity so that the user
	 * can see the status of the connections for a while after
	 * they are finished.
	 */
	public void freeConnectionListener(Connection connection)
	{
		//
		// Get the close delay.
		//
		int closeDelay = Settings.getCloseDelay();

		//
		// If there is a close delay time then sleep a while.
		//
		if (closeDelay > 0)
		{
			try
			{
				Thread.sleep(closeDelay * 1000);
			}
			catch (Exception e)
			{
			}
		}

		//
		// Now proceed to close down the display.
		//
		SwingListener listener =
			(SwingListener) connection.getListener();

		mModel.removeConnection(listener);

		super.freeConnectionListener(connection);

		//
		// Update the display to remove the row.
		//
		changed(null);
	}


	/**
	 * Allocate a connection listener to listen on a connection.
	 * We construct a new one for each connection.  The listener
	 * is used to manage one row of our connection table.
	 */
	protected ConnectionListener
		allocateConnectionListener()
	{
		SwingListener listener = new SwingListener();

		mModel.addConnection(listener);

		return listener;
	}
}
